@@ -8,11 +8,11 @@ pub mod parser;
 pub mod random;
 mod types;
 
-use std::fmt;
+use std::{fmt, str::FromStr};
 
 use colorspace::ColorSpace;
 pub use helper::Fraction;
-use helper::{clamp, interpolate, interpolate_angle, mod_positive};
+use helper::{clamp, interpolate, interpolate_angle, mod_positive, MaxPrecision};
 use types::{Hue, Scalar};
 
 /// The representation of a color.
@@ -99,8 +99,8 @@ impl Color {
     /// sRGB range.
     ///
     /// See:
-    /// - https://en.wikipedia.org/wiki/CIE_1931_color_space
-    /// - https://en.wikipedia.org/wiki/SRGB
+    /// - <https://en.wikipedia.org/wiki/CIE_1931_color_space>
+    /// - <https://en.wikipedia.org/wiki/SRGB>
     pub fn from_xyz(x: Scalar, y: Scalar, z: Scalar, alpha: Scalar) -> Color {
         Self::from(&XYZ { x, y, z, alpha })
     }
@@ -114,7 +114,7 @@ impl Color {
     /// Create a `Color` from L, a and b coordinates coordinates in the Lab color
     /// space. Note: See documentation for `from_xyz`. The same restrictions apply here.
     ///
-    /// See: https://en.wikipedia.org/wiki/Lab_color_space
+    /// See: <https://en.wikipedia.org/wiki/Lab_color_space>
     pub fn from_lab(l: Scalar, a: Scalar, b: Scalar, alpha: Scalar) -> Color {
         Self::from(&Lab { l, a, b, alpha })
     }
@@ -123,7 +123,7 @@ impl Color {
     /// This is a cylindrical transform of the Lab color space. Note: See documentation for
     /// `from_xyz`. The same restrictions apply here.
     ///
-    /// See: https://en.wikipedia.org/wiki/Lab_color_space
+    /// See: <https://en.wikipedia.org/wiki/Lab_color_space>
     pub fn from_lch(l: Scalar, c: Scalar, h: Scalar, alpha: Scalar) -> Color {
         Self::from(&LCh { l, c, h, alpha })
     }
@@ -141,14 +141,30 @@ impl Color {
         HSLA::from(self)
     }
 
-    /// Format the color as a HSL-representation string (`hsl(123, 50.3%, 80.1%)`).
+    /// Format the color as a HSL-representation string (`hsla(123, 50.3%, 80.1%, 0.4)`). If the
+    /// alpha channel is `1.0`, the simplified `hsl()` format will be used instead.
     pub fn to_hsl_string(&self, format: Format) -> String {
+        let space = if format == Format::Spaces { " " } else { "" };
+        let (a_prefix, a) = if self.alpha == 1.0 {
+            ("", "".to_string())
+        } else {
+            (
+                "a",
+                format!(
+                    ",{space}{alpha}",
+                    alpha = MaxPrecision::wrap(3, self.alpha),
+                    space = space
+                ),
+            )
+        };
         format!(
-            "hsl({:.0},{space}{:.1}%,{space}{:.1}%)",
-            self.hue.value(),
-            100.0 * self.saturation,
-            100.0 * self.lightness,
-            space = if format == Format::Spaces { " " } else { "" }
+            "hsl{a_prefix}({h:.0},{space}{s:.1}%,{space}{l:.1}%{a})",
+            space = space,
+            a_prefix = a_prefix,
+            h = self.hue.value(),
+            s = 100.0 * self.saturation,
+            l = 100.0 * self.lightness,
+            a = a,
         )
     }
 
@@ -158,15 +174,31 @@ impl Color {
         RGBA::<u8>::from(self)
     }
 
-    /// Format the color as a RGB-representation string (`rgb(255, 127,  0)`).
+    /// Format the color as a RGB-representation string (`rgba(255, 127, 0, 0.5)`). If the alpha channel
+    /// is `1.0`, the simplified `rgb()` format will be used instead.
     pub fn to_rgb_string(&self, format: Format) -> String {
         let rgba = RGBA::<u8>::from(self);
+        let space = if format == Format::Spaces { " " } else { "" };
+        let (a_prefix, a) = if self.alpha == 1.0 {
+            ("", "".to_string())
+        } else {
+            (
+                "a",
+                format!(
+                    ",{space}{alpha}",
+                    alpha = MaxPrecision::wrap(3, rgba.alpha),
+                    space = space
+                ),
+            )
+        };
         format!(
-            "rgb({r},{space}{g},{space}{b})",
+            "rgb{a_prefix}({r},{space}{g},{space}{b}{a})",
+            space = space,
+            a_prefix = a_prefix,
             r = rgba.r,
             g = rgba.g,
             b = rgba.b,
-            space = if format == Format::Spaces { " " } else { "" }
+            a = a,
         )
     }
 
@@ -189,27 +221,49 @@ impl Color {
         )
     }
 
-    /// Format the color as a floating point RGB-representation string (`rgb(1.0, 0.5,  0)`).
+    /// Format the color as a floating point RGB-representation string (`rgb(1.0, 0.5, 0)`). If the alpha channel
+    /// is `1.0`, the simplified `rgb()` format will be used instead.
     pub fn to_rgb_float_string(&self, format: Format) -> String {
         let rgba = RGBA::<f64>::from(self);
+        let space = if format == Format::Spaces { " " } else { "" };
+        let (a_prefix, a) = if self.alpha == 1.0 {
+            ("", "".to_string())
+        } else {
+            (
+                "a",
+                format!(
+                    ",{space}{alpha}",
+                    alpha = MaxPrecision::wrap(3, rgba.alpha),
+                    space = space
+                ),
+            )
+        };
         format!(
-            "rgb({r:.3},{space}{g:.3},{space}{b:.3})",
+            "rgb{a_prefix}({r:.3},{space}{g:.3},{space}{b:.3}{a})",
+            space = space,
+            a_prefix = a_prefix,
             r = rgba.r,
             g = rgba.g,
             b = rgba.b,
-            space = if format == Format::Spaces { " " } else { "" }
+            a = a,
         )
     }
 
-    /// Format the color as a RGB-representation string (`#fc0070`).
+    /// Format the color as a RGB-representation string (`#fc0070`). The output will contain 6 hex
+    /// digits if the alpha channel is `1.0`, or 8 hex digits otherwise.
     pub fn to_rgb_hex_string(&self, leading_hash: bool) -> String {
         let rgba = self.to_rgba();
         format!(
-            "{}{:02x}{:02x}{:02x}",
+            "{}{:02x}{:02x}{:02x}{}",
             if leading_hash { "#" } else { "" },
             rgba.r,
             rgba.g,
-            rgba.b
+            rgba.b,
+            if rgba.alpha == 1.0 {
+                "".to_string()
+            } else {
+                format!("{:02x}", (rgba.alpha * 255.).round() as u8)
+            }
         )
     }
 
@@ -228,15 +282,15 @@ impl Color {
     /// Get XYZ coordinates according to the CIE 1931 color space.
     ///
     /// See:
-    /// - https://en.wikipedia.org/wiki/CIE_1931_color_space
-    /// - https://en.wikipedia.org/wiki/SRGB
+    /// - <https://en.wikipedia.org/wiki/CIE_1931_color_space>
+    /// - <https://en.wikipedia.org/wiki/SRGB>
     pub fn to_xyz(&self) -> XYZ {
         XYZ::from(self)
     }
 
     /// Get coordinates according to the LSM color space
     ///
-    /// See https://en.wikipedia.org/wiki/LMS_color_space for info on the color space as well as an
+    /// See <https://en.wikipedia.org/wiki/LMS_color_space> for info on the color space as well as an
     /// algorithm for converting from CIE XYZ
     pub fn to_lms(&self) -> LMS {
         LMS::from(self)
@@ -244,39 +298,61 @@ impl Color {
 
     /// Get L, a and b coordinates according to the Lab color space.
     ///
-    /// See: https://en.wikipedia.org/wiki/Lab_color_space
+    /// See: <https://en.wikipedia.org/wiki/Lab_color_space>
     pub fn to_lab(&self) -> Lab {
         Lab::from(self)
     }
 
-    /// Format the color as a Lab-representation string (`Lab(41, 83, -93)`).
+    /// Format the color as a Lab-representation string (`Lab(41, 83, -93, 0.5)`). If the alpha channel
+    /// is `1.0`, it won't be included in the output.
     pub fn to_lab_string(&self, format: Format) -> String {
         let lab = Lab::from(self);
+        let space = if format == Format::Spaces { " " } else { "" };
         format!(
-            "Lab({l:.0},{space}{a:.0},{space}{b:.0})",
+            "Lab({l:.0},{space}{a:.0},{space}{b:.0}{alpha})",
             l = lab.l,
             a = lab.a,
             b = lab.b,
-            space = if format == Format::Spaces { " " } else { "" }
+            space = space,
+            alpha = if self.alpha == 1.0 {
+                "".to_string()
+            } else {
+                format!(
+                    ",{space}{alpha}",
+                    alpha = MaxPrecision::wrap(3, self.alpha),
+                    space = space
+                )
+            }
         )
     }
 
     /// Get L, C and h coordinates according to the CIE LCh color space.
     ///
-    /// See: https://en.wikipedia.org/wiki/Lab_color_space
+    /// See: <https://en.wikipedia.org/wiki/Lab_color_space>
     pub fn to_lch(&self) -> LCh {
         LCh::from(self)
     }
 
-    /// Format the color as a LCh-representation string (`LCh(0.3, 0.2, 0.1)`).
+    /// Format the color as a LCh-representation string (`LCh(0.3, 0.2, 0.1, 0.5)`). If the alpha channel
+    /// is `1.0`, it won't be included in the output.
     pub fn to_lch_string(&self, format: Format) -> String {
         let lch = LCh::from(self);
+        let space = if format == Format::Spaces { " " } else { "" };
         format!(
-            "LCh({l:.0},{space}{c:.0},{space}{h:.0})",
+            "LCh({l:.0},{space}{c:.0},{space}{h:.0}{alpha})",
             l = lch.l,
             c = lch.c,
             h = lch.h,
-            space = if format == Format::Spaces { " " } else { "" }
+            space = space,
+            alpha = if self.alpha == 1.0 {
+                "".to_string()
+            } else {
+                format!(
+                    ",{space}{alpha}",
+                    alpha = MaxPrecision::wrap(3, self.alpha),
+                    space = space
+                )
+            }
         )
     }
 
@@ -458,7 +534,7 @@ impl Color {
 
     /// The percieved brightness of the color (A number between 0.0 and 1.0).
     ///
-    /// See: https://www.w3.org/TR/AERT#color-contrast
+    /// See: <https://www.w3.org/TR/AERT#color-contrast>
     pub fn brightness(&self) -> Scalar {
         let c = self.to_rgba_float();
         (299.0 * c.r + 587.0 * c.g + 114.0 * c.b) / 1000.0
@@ -473,7 +549,7 @@ impl Color {
     /// The relative brightness of a color (normalized to 0.0 for darkest black
     /// and 1.0 for lightest white), according to the WCAG definition.
     ///
-    /// See: https://www.w3.org/TR/2008/REC-WCAG20-20081211/#relativeluminancedef
+    /// See: <https://www.w3.org/TR/2008/REC-WCAG20-20081211/#relativeluminancedef>
     pub fn luminance(&self) -> Scalar {
         fn f(s: Scalar) -> Scalar {
             if s <= 0.03928 {
@@ -495,7 +571,7 @@ impl Color {
     /// to 21.0. Two colors with a contrast ratio of 4.5 or higher can be used as text color and
     /// background color and should be well readable.
     ///
-    /// https://www.w3.org/TR/2008/REC-WCAG20-20081211/#contrast-ratiodef
+    /// <https://www.w3.org/TR/2008/REC-WCAG20-20081211/#contrast-ratiodef>
     pub fn contrast_ratio(&self, other: &Color) -> Scalar {
         let l_self = self.luminance();
         let l_other = other.luminance();
@@ -527,7 +603,7 @@ impl Color {
     /// Compute the perceived 'distance' between two colors according to the CIE76 delta-E
     /// standard. A distance below ~2.3 is not noticable.
     ///
-    /// See: https://en.wikipedia.org/wiki/Color_difference
+    /// See: <https://en.wikipedia.org/wiki/Color_difference>
     pub fn distance_delta_e_cie76(&self, other: &Color) -> Scalar {
         delta_e::cie76(&self.to_lab(), &other.to_lab())
     }
@@ -535,7 +611,7 @@ impl Color {
     /// Compute the perceived 'distance' between two colors according to the CIEDE2000 delta-E
     /// standard.
     ///
-    /// See: https://en.wikipedia.org/wiki/Color_difference
+    /// See: <https://en.wikipedia.org/wiki/Color_difference>
     pub fn distance_delta_e_ciede2000(&self, other: &Color) -> Scalar {
         delta_e::ciede2000(&self.to_lab(), &other.to_lab())
     }
@@ -547,12 +623,42 @@ impl Color {
             .mix(&C::from_color(other), fraction)
             .into_color()
     }
+
+    /// Alpha composite two colors, placing the second over the first.
+    pub fn composite(&self, source: &Color) -> Color {
+        let backdrop = self.to_rgba();
+        let source = source.to_rgba();
+
+        // Composite A over B (see https://en.wikipedia.org/wiki/Alpha_compositing)
+        //
+        //   αo = αa + αb(1 - αa)
+        //
+        //        Ca * αa + Cb * αb(1 - αa)
+        //   Co = -------------------------
+        //                   αo
+        //
+        //       αo:  output alpha
+        //   αa, αb:  A/B alpha
+        //       Co:  output color
+        //   Ca, Cb:  A/B color
+        //
+        fn composite_channel(c_a: u8, a_a: f64, c_b: u8, a_b: f64, a_o: f64) -> u8 {
+            ((c_a as f64 * a_a + c_b as f64 * a_b * (1.0 - a_a)) / a_o).floor() as u8
+        }
+
+        let a = source.alpha + backdrop.alpha * (1.0 - source.alpha);
+        let r = composite_channel(source.r, source.alpha, backdrop.r, backdrop.alpha, a);
+        let g = composite_channel(source.g, source.alpha, backdrop.g, backdrop.alpha, a);
+        let b = composite_channel(source.b, source.alpha, backdrop.b, backdrop.alpha, a);
+
+        Color::from_rgba(r, g, b, a)
+    }
 }
 
 // by default Colors will be printed into HSLA fromat
 impl fmt::Display for Color {
     fn fmt(&self, f: &mut fmt::Formatter<'_>) -> fmt::Result {
-        write!(f, "{}", HSLA::from(self).to_string())
+        write!(f, "{}", HSLA::from(self))
     }
 }
 
@@ -568,6 +674,14 @@ impl PartialEq for Color {
     }
 }
 
+impl FromStr for Color {
+    type Err = &'static str;
+
+    fn from_str(s: &str) -> Result<Self, Self::Err> {
+        parser::parse_color(s).ok_or("invalid color string")
+    }
+}
+
 impl From<&HSLA> for Color {
     fn from(color: &HSLA) -> Self {
         Color {
@@ -744,7 +858,7 @@ impl ColorSpace for RGBA<f64> {
         c.to_rgba_float()
     }
 
-    fn into_color(&self) -> Color {
+    fn into_color(self) -> Color {
         Color::from_rgba_float(self.r, self.g, self.b, self.alpha)
     }
 
@@ -765,17 +879,18 @@ impl From<&Color> for RGBA<f64> {
         let m = color.lightness - chr / 2.0;
         let x = chr * (1.0 - Scalar::abs(h_s % 2.0 - 1.0));
 
+        #[allow(clippy::upper_case_acronyms)]
         struct RGB(Scalar, Scalar, Scalar);
 
         let col = if h_s < 1.0 {
             RGB(chr, x, 0.0)
-        } else if 1.0 <= h_s && h_s < 2.0 {
+        } else if (1.0..2.0).contains(&h_s) {
             RGB(x, chr, 0.0)
-        } else if 2.0 <= h_s && h_s < 3.0 {
+        } else if (2.0..3.0).contains(&h_s) {
             RGB(0.0, chr, x)
-        } else if 3.0 <= h_s && h_s < 4.0 {
+        } else if (3.0..4.0).contains(&h_s) {
             RGB(0.0, x, chr)
-        } else if 4.0 <= h_s && h_s < 5.0 {
+        } else if (4.0..5.0).contains(&h_s) {
             RGB(x, 0.0, chr)
         } else {
             RGB(chr, 0.0, x)
@@ -831,7 +946,7 @@ impl ColorSpace for HSLA {
         c.to_hsla()
     }
 
-    fn into_color(&self) -> Color {
+    fn into_color(self) -> Color {
         Color::from_hsla(self.h, self.s, self.l, self.alpha)
     }
 
@@ -950,7 +1065,7 @@ impl ColorSpace for Lab {
         c.to_lab()
     }
 
-    fn into_color(&self) -> Color {
+    fn into_color(self) -> Color {
         Color::from_lab(self.l, self.a, self.b, self.alpha)
     }
 
@@ -1011,7 +1126,7 @@ impl ColorSpace for LCh {
         c.to_lch()
     }
 
-    fn into_color(&self) -> Color {
+    fn into_color(self) -> Color {
         Color::from_lch(self.l, self.c, self.h, self.alpha)
     }
 
@@ -1153,7 +1268,7 @@ impl ColorScale {
                     .iter()
                     .position(|c| position.value() < c.position.value());
 
-                let index = next_index.unwrap_or_else(|| self.color_stops.len());
+                let index = next_index.unwrap_or(self.color_stops.len());
 
                 let color_stop = ColorStop { color, position };
 
@@ -1668,4 +1783,19 @@ mod tests {
         let c3 = Color::from_rgb(143, 111, 76);
         assert_eq!("cmyk(0, 22, 47, 44)", c3.to_cmyk_string(Format::Spaces));
     }
+
+    #[test]
+    fn alpha_roundtrip_hex_to_decimal() {
+        // We use a max of 3 decimal places when displaying RGB floating point
+        // alpha values. This test insures that is sufficient to "roundtrip"
+        // from hex (0 < n < 255) to float (0 < n < 1) and back again,
+        // e.g. hex `80` is float `0.502`, which parses to hex `80`, and so on.
+        for alpha_int in 0..255 {
+            let hex_string = format!("#000000{:02x}", alpha_int);
+            let parsed_from_hex = hex_string.parse::<Color>().unwrap();
+            let rgba_string = parsed_from_hex.to_rgb_float_string(Format::Spaces);
+            let parsed_from_rgba = rgba_string.parse::<Color>().unwrap();
+            assert_eq!(hex_string, parsed_from_rgba.to_rgb_hex_string(true));
+        }
+    }
 }
